@@ -5,11 +5,11 @@ use warnings;
 
 =head1 VERSION
 
-Version 1.002000
+Version 1.003000
 
 =cut
 
-our $VERSION = '1.002001';
+our $VERSION = '1.003000';
 
 =head1 NAME
 
@@ -28,8 +28,15 @@ Catalyst::Controller::DBIC::API
       update_allows => ['name', 'age', 'nickname'], # columns that update allows
       update_allows => ['name', 'age', 'nickname'], # columns that update allows
       list_returns => [qw/name age/], # columns that list returns
+      list_prefetch => ['cds'], # relationships that are prefetched when no prefetch param is passed
+      list_prefetch_allows => [ # every possible prefetch param allowed
+          'cds',
+          qw/ cds /,
+          { cds => 'tracks' },
+          { cds => [qw/ tracks /] }
+      ],
       list_ordered_by => [qw/age/], # order of generated list
-      list_search_exposes => [qw/age nickname/, { cd => [qw/title year/] }], # columns that can be searched on via list
+      list_search_exposes => [qw/age nickname/, { cds => [qw/title year/] }], # columns that can be searched on via list
       );
 
   # Provides the following functional endpoints:
@@ -68,11 +75,13 @@ The class, create_requires, create_allows and update_requires parameters can als
       return;
     }
 
-    $self->NEXT::setup($c);
+    $self->next::method($c);
   }
 
 Generally it's better to have one controller for each DBIC source with the config hardcoded, but in some cases this isn't possible.
 
+Note that the Chained, CaptureArgs and PathPart are just standard Catalyst configuration parameters and that then endpoint specified in Chained - in this case '/api/rpc/rpc_base' - must actually exist elsewhere in your application. See L<Catalyst::DispatchType::Chained> for more details.
+
 =head2 class
 
 Whatever you would pass to $c->model to get a resultset for this class. MyAppDB::Track for example.
@@ -93,6 +102,20 @@ Arrayref listing columns that update will allow. Columns passed to update that a
 
 Arguments to pass to L<DBIx::Class::ResultSet/select> when performing search for L</list>.
 
+=head2 list_prefetch
+
+Arguments to pass to L<DBIx::Class::ResultSet/prefetch> when performing search for L</list>.
+
+=head2 list_prefetch_allows
+
+Arrayref listing relationships that are allowed to be prefetched.
+This is necessary to avoid denial of service attacks in form of
+queries which would return a large number of data
+and unwanted disclosure of data.
+Every element of the arrayref is one allowed parameter to prefetch.
+So for three searches, all requiring different prefetch parameters,
+three elements have to be passed to list_prefetch_allows in the controller.
+
 =head2 list_grouped_by
 
 Arguments to pass to L<DBIx::Class::ResultSet/group_by> when performing search for L</list>.
@@ -210,7 +233,7 @@ This action is the chain root of the controller. It must either be overridden or
   sub setup :Chained('/api/rpc_base') :CaptureArgs(0) :PathPart('track') {
     my ($self, $c) = @_;
 
-    $self->NEXT::setup($c);
+    $self->next::method($c);
   }
 
 This action will populate $c->stash->{$self->rs_stash_key} with $c->model($self->class) for other actions in the chain to use.
@@ -285,7 +308,7 @@ By default the create, delete and update actions will not return anything apart
     my ($self, $c) = @_;
 
     # will set $c->stash->{created_object} if successful
-    $self->NEXT::create($c);
+    $self->next::method($c);
 
     if ($c->stash->{created_object}) {    
       # $c->stash->{response} will be serialized in the end action
@@ -313,7 +336,7 @@ Similarly you might want create, update and delete to all forward to the list ac
 
   Zbigniew Lukasiak <zzbbyy@gmail.com>
 
-  Alexander Hartmaier <alex_hartmaier@hotmail.com>
+  Alexander Hartmaier <abraxxa@cpan.org>
 
 =head1 SPECIAL THANKS
 
